/*
Mihalea Geanina 334 CB
*/

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdint.h>

#define RED 0
#define GREEN 1
#define EMPTY 9

#define UP		0
#define RIGHT	1
#define DOWN	2
#define LEFT	3

#define NR_OF_ELEM	3
#define MAX_MOVES	9

typedef struct Led {
	uint8_t pos[2];
	volatile uint8_t *port[2];
} Led;

Led matrix[NR_OF_ELEM][NR_OF_ELEM];
uint8_t board[NR_OF_ELEM][NR_OF_ELEM];

int currentX = 0;
int currentY = 0;

void init()
{

	DDRA = 0xFF;
	DDRC = 0xFF;
	DDRD = 0xFF;
	DDRB = 0x00;

	DDRB |= (1<<PB7);
	DDRB |= (1<<PB6);

	PORTA = 0;
	PORTC = 0;
	PORTD = 0;

	PORTB |= (1<<PB1);
	PORTB |= (1<<PB2);
	PORTB |= (1<<PB3);
	PORTB |= (1<<PB4);
	PORTB |= (1<<PB5);
}


void closePorts() {

	PORTA = 0;
	PORTC = 0;
	PORTD = 0;
	PORTB &= (~(1<<PB7));
	PORTB &= (~(1<<PB6));
}

void lightOffAll(int color){

	int i;
	int j;

	for (i = 0; i < NR_OF_ELEM; ++i)
		for (j = 0; j < NR_OF_ELEM; ++j){

			*(matrix[i][j].port[RED]) &= ~(1 << matrix[i][j].pos[RED]);
			*(matrix[i][j].port[GREEN]) &= ~(1 << matrix[i][j].pos[GREEN]);
		}

}

void lightOnAll(int color) {

	int i;
	int j;

	lightOffAll(RED);
	lightOffAll(GREEN);

	for (i = 0; i < NR_OF_ELEM; ++i)
		for (j = 0; j < NR_OF_ELEM; ++j)
			*(matrix[i][j].port[color]) |= (1 << matrix[i][j].pos[color]);
}

void lightOn(int i, int j, int color) {

	*(matrix[i][j].port[color]) |= (1 << matrix[i][j].pos[color]);
}

int buttonIsPressed(int pinNr) {

	if (bit_is_clear(PINB, pinNr)) {

		_delay_ms(25);

		if (bit_is_clear(PINB, pinNr))
			return 1;
	}

	return 0;
}

void checkNextLed(int *currentX, int *currentY) {

	int i, j;

	for(i = 0; i < NR_OF_ELEM; ++i)
		for (j = 0; j < NR_OF_ELEM; ++j)

			if (board[i][j] == EMPTY) {

				*currentX = i;
				*currentY = j;

				return;
			}
}

void freeBoard(){

	int i, j;

	for (i = 0; i < NR_OF_ELEM; ++i)
		for (j = 0; j < NR_OF_ELEM; ++j)

			board[i][j] = EMPTY;
}


void findPosition(int pos, int *currentX, int *currentY, int playerMove) {

	int i;
	int oppositePlayer = 1 - playerMove;

	switch(pos) {

		case RIGHT:

			for (i = 0; i< NR_OF_ELEM; ++i) {

				++(*currentX);

				if (*currentX > NR_OF_ELEM - 1)
					*currentX = 0;

				if (board[*currentX][*currentY] == EMPTY)
					break;

				else
					if(board[*currentX][*currentY] == oppositePlayer && (*currentX) + 1 == 2)
						*currentX = 2;

			}
			break;

		case LEFT:
			for (i = 0; i< NR_OF_ELEM; ++i) {

				(*currentX)--;

				if (*currentX < 0)
					*currentX = 2;

				if (board[*currentX][*currentY] == EMPTY)
					break;

				else 
					if(board[*currentX][*currentY] == oppositePlayer
						&& (*currentX) - 1 == 0)
						*currentX = 0;
			}
			break;

		case UP:
			for (i = 0; i< NR_OF_ELEM; ++i) {

				(*currentY)--;

				if (*currentY < 0)
					*currentY = 2;

				if (board[*currentX][*currentY] == EMPTY)
					break;

				else if(board[*currentX][*currentY] == oppositePlayer
						&& (*currentY) - 1 == 0)
						*currentY = 0;
			}
			break;

		case DOWN:
			for (i = 0; i< NR_OF_ELEM; i++) {

				++(*currentY);

				if (*currentY > NR_OF_ELEM - 1)
					*currentY = 0;

				if (board[*currentX][*currentY] == EMPTY)
					break;

				else if(board[*currentX][*currentY] == oppositePlayer
						&& (*currentY) + 1 == 2)
						*currentY = 2;
			}
			break;
	}

}

void buttonHandler(int playerMove, int x){

	*(matrix[currentX][currentY].port[playerMove]) &= ~(1 << matrix[currentX][currentY].pos[playerMove]);

	if(x == 0)
		findPosition(UP, &currentX, &currentY, playerMove);
	else
		if(x == 1)
			findPosition(LEFT, &currentX, &currentY, playerMove);
	else
		if(x == 2)
			findPosition(RIGHT, &currentX, &currentY, playerMove);
	else
		if(x == 3)
			findPosition(DOWN, &currentX, &currentY, playerMove);

	*(matrix[currentX][currentY].port[playerMove]) |= (1 << matrix[currentX][currentY].pos[playerMove]);

	_delay_ms(200);
}


int checkWinner(int currentX, int currentY, int color) {

	if(board[currentX][0] == color &&
		board[currentX][1] ==  color && board[currentX][NR_OF_ELEM - 1] == color)
		return 1;

	else
		if (board[0][currentY] == color &&
			board[1][currentY] ==  color && board[NR_OF_ELEM - 1][currentY] == color)
			return 1;

	else
		if(board[0][0] == color &&
			board[1][1] ==  color && board[NR_OF_ELEM - 1][NR_OF_ELEM - 1] == color)
			return 1;

	else
		if(board[0][NR_OF_ELEM - 1] == color &&
			board[1][1] ==  color && board[NR_OF_ELEM - 1][0] == color)
			return 1;

	return 0;
}

void animate_winner(int playerMove) {

	int oppisitePlayer = 1 - playerMove;

	lightOffAll(oppisitePlayer);
	_delay_ms(300);
	lightOnAll(playerMove);
	_delay_ms(300);

	lightOffAll(oppisitePlayer);
	_delay_ms(300);
	lightOnAll(playerMove);
	_delay_ms(300);

	_delay_ms(200);
	lightOffAll(oppisitePlayer);
	_delay_ms(200);
	lightOffAll(playerMove);


	int i;

	for(i = 0; i < 3; ++i){
		lightOn(0, i, playerMove);
		_delay_ms(200);
	}

	for(i = 2; i >= 0; --i){
		lightOn(1, i, playerMove);
		_delay_ms(200);
	}

	for(i = 0; i < 3; ++i){
		lightOn(2, i, playerMove);
		_delay_ms(200);
	}

	_delay_ms(200);
	lightOffAll(oppisitePlayer);
	_delay_ms(200);
	lightOffAll(playerMove);

	for(i = 0; i < 3; ++i){
		lightOn(i, 0, playerMove);
		_delay_ms(200);
	}

	for(i = 2; i >= 0; --i){
		lightOn(i, 1, playerMove);
		_delay_ms(200);
	}

	for(i = 0; i < 3; ++i){
		lightOn(i, 2, playerMove);
		_delay_ms(200);
	}
}

void animate_init() {

	_delay_ms(200);
	lightOn(0,0,RED);
	_delay_ms(200);
	lightOn(1,1,RED);
	_delay_ms(200);
	lightOn(2,2,RED);
	_delay_ms(200);
	lightOn(2,0,RED);
	_delay_ms(200);
	lightOn(0,2,RED);
	_delay_ms(300);

	lightOffAll(RED);

	_delay_ms(200);

	lightOn(0,0,GREEN);
	_delay_ms(200);
	lightOn(0,1,GREEN);
	_delay_ms(200);
	lightOn(0,2,GREEN);
	_delay_ms(200);

	lightOn(1,2,GREEN);
	_delay_ms(200);
	lightOn(2,2,GREEN);
	_delay_ms(200);
	lightOn(2,1,GREEN);
	_delay_ms(200);
	lightOn(2,0,GREEN);
	_delay_ms(200);
	lightOn(1,0,GREEN);
	_delay_ms(400);

	lightOffAll(GREEN);
}

void animate_draw(playerMove) {

	_delay_ms(300);
	lightOffAll(1 - playerMove);
	lightOffAll(playerMove);
	_delay_ms(300);

	lightOnAll(playerMove);
	_delay_ms(300);
	lightOffAll(playerMove);
	_delay_ms(300);

	lightOnAll(1 - playerMove);
	_delay_ms(300);
	lightOffAll(1 - playerMove);
	_delay_ms(300);

	int i;

	for(i = 0; i < 3; ++i){
		lightOn(0, i, RED);
		_delay_ms(200);
	}

	for(i = 2; i >= 0; --i){
		lightOn(1, i, GREEN);
		_delay_ms(200);
	}

	for(i = 0; i < 3; ++i){
		lightOn(2, i, RED);
		_delay_ms(200);
	}

	_delay_ms(200);
	lightOffAll(1 - playerMove);
	_delay_ms(200);
	lightOffAll(playerMove);

	for(i = 0; i < 3; ++i){
		lightOn(i, 0, RED);
		_delay_ms(200);
	}

	for(i = 2; i >= 0; --i) {
		lightOn(i, 1, GREEN);
		_delay_ms(200);
	}

	for(i = 0; i < 3; ++i) {
		lightOn(i, 2, RED);
		_delay_ms(200);
	}

}

void light_on_off(int playerMove) {

	*(matrix[currentX][currentY].port[playerMove]) ^= (1 << matrix[currentX][currentY].pos[playerMove]);
	_delay_ms(200);
}

void play() {

	int playerMove = RED;
	int noOfMoves = 0;

	while (1) {

		if (buttonIsPressed(PB4)) {

			buttonHandler(playerMove, 0);
			continue;
		}

		if (buttonIsPressed(PB5)) {

			buttonHandler(playerMove, 1);
			continue;
		}

		if (buttonIsPressed(PB2)) {

			*(matrix[currentX][currentY].port[playerMove]) |= (1 << matrix[currentX][currentY].pos[playerMove]);
			board[currentX][currentY] = playerMove;
			++noOfMoves;

			if (noOfMoves >= 5 && checkWinner(currentX, currentY, playerMove)) {

				animate_winner(playerMove);
				return;
			}

			if (noOfMoves == MAX_MOVES){

				animate_draw(playerMove);
				return;
			}

			checkNextLed(&currentX, &currentY);
			playerMove = 1 - playerMove;
		}

		if (buttonIsPressed(PB1)) {

			buttonHandler(playerMove, 2);
			continue;
		}

		if (buttonIsPressed(PB3)) {

			buttonHandler(playerMove, 3);
			continue;
		}

		light_on_off(playerMove);
	}
}

void initializeBoard() {

	matrix[0][0].pos[GREEN] = PA1;
	matrix[0][0].port[GREEN] = &PORTA;
	matrix[0][0].pos[RED] = PA0;
	matrix[0][0].port[RED] = &PORTA;

	matrix[0][1].pos[GREEN] = PA5;
	matrix[0][1].port[GREEN] = &PORTA;
	matrix[0][1].pos[RED] = PA4;
	matrix[0][1].port[RED] = &PORTA;

	matrix[0][2].pos[GREEN] = PD0;
	matrix[0][2].port[GREEN] = &PORTD;
	matrix[0][2].pos[RED] = PD1;
	matrix[0][2].port[RED] = &PORTD;

	matrix[1][0].pos[GREEN] = PA3;
	matrix[1][0].port[GREEN] = &PORTA;
	matrix[1][0].pos[RED] = PA2;
	matrix[1][0].port[RED] = &PORTA;

	matrix[1][1].pos[GREEN] = PC1;
	matrix[1][1].port[GREEN] = &PORTC;
	matrix[1][1].pos[RED] = PC0;
	matrix[1][1].port[RED] = &PORTC;

	matrix[1][2].pos[GREEN] = PC7;
	matrix[1][2].port[GREEN] = &PORTC;
	matrix[1][2].pos[RED] = PC6;
	matrix[1][2].port[RED] = &PORTC;

	matrix[2][0].pos[GREEN] = PA7;
	matrix[2][0].port[GREEN] = &PORTA;
	matrix[2][0].pos[RED] = PA6;
	matrix[2][0].port[RED] = &PORTA;

	matrix[2][1].pos[GREEN] = PD5;
	matrix[2][1].port[GREEN] = &PORTD;
	matrix[2][1].pos[RED] = PD6;
	matrix[2][1].port[RED] = &PORTD;

	matrix[2][2].pos[GREEN] = PB7;
	matrix[2][2].port[GREEN] = &PORTB;
	matrix[2][2].pos[RED] = PB6;
	matrix[2][2].port[RED] = &PORTB;
}

int main()
{

	init();
	initializeBoard();

	animate_init();

	while(1) {

		freeBoard();
		play();

		while (!buttonIsPressed(PB2));

		closePorts();
		freeBoard();

		_delay_ms(300);
	}

	return 0;
}
